@@ -1629,28 +1629,7 @@ gdk_directfb_translate_key (DFBInputDeviceKeyIdentifier key_id,
 				break;
 
 			case DIKT_DEAD:
-				switch (key_symbol)
-				{
-					case DIKS_DEAD_ABOVEDOT:     keyval = GDK_dead_abovedot;     break;
-					case DIKS_DEAD_ABOVERING:    keyval = GDK_dead_abovering;    break;
-					case DIKS_DEAD_ACUTE:        keyval = GDK_dead_acute;        break;
-					case DIKS_DEAD_BREVE:        keyval = GDK_dead_breve;        break;
-					case DIKS_DEAD_CARON:        keyval = GDK_dead_caron;        break;
-					case DIKS_DEAD_CEDILLA:      keyval = GDK_dead_cedilla;      break;
-					case DIKS_DEAD_CIRCUMFLEX:   keyval = GDK_dead_circumflex;   break;
-					case DIKS_DEAD_DIAERESIS:    keyval = GDK_dead_diaeresis;    break;
-					case DIKS_DEAD_DOUBLEACUTE:  keyval = GDK_dead_doubleacute;  break;
-					case DIKS_DEAD_GRAVE:        keyval = GDK_dead_grave;        break;
-					case DIKS_DEAD_IOTA:         keyval = GDK_dead_iota;         break;
-					case DIKS_DEAD_MACRON:       keyval = GDK_dead_macron;       break;
-					case DIKS_DEAD_OGONEK:       keyval = GDK_dead_ogonek;       break;
-					case DIKS_DEAD_SEMIVOICED_SOUND:
-								     keyval = GDK_dead_semivoiced_sound;                        break;
-					case DIKS_DEAD_TILDE:        keyval = GDK_dead_tilde;        break;
-					case DIKS_DEAD_VOICED_SOUND: keyval = GDK_dead_voiced_sound; break;
-					default:
-								     break;
-				}
+				/* dead keys are handled directly by directfb */
 				break;
 
 			case DIKT_CUSTOM:
@@ -1715,7 +1694,7 @@ _gdk_directfb_keyboard_exit (void)
 
 void
 gdk_directfb_translate_key_event (DFBWindowEvent *dfb_event,
-		GdkEventKey    *event)
+                                  GdkEventKey    *event)
 {
 	gint  len;
 	gchar buf[6];
@@ -1753,6 +1732,32 @@ gdk_directfb_translate_key_event (DFBWindowEvent *dfb_event,
 	event->length = len;
 }
 
+/**
+ * gdk_keymap_get_caps_lock_state:
+ * @keymap: a #GdkKeymap
+ *
+ * Returns whether the Caps Lock modifer is locked.
+ *
+ * Returns: %TRUE if Caps Lock is on
+ *
+ * Since: 2.16
+ */
+gboolean
+gdk_keymap_get_caps_lock_state (GdkKeymap *keymap)
+{
+  IDirectFBInputDevice *keyboard = _gdk_display->keyboard;
+
+  if (keyboard)
+    {
+      DFBInputDeviceLockState  state;
+
+      if (keyboard->GetLockState (keyboard, &state) == DFB_OK)
+        return ((state & DILS_CAPS) != 0);
+    }
+
+  return FALSE;
+}
+
 /**
  * gdk_keymap_get_entries_for_keycode:
  * @keymap: a #GdkKeymap or %NULL to use the default keymap
