@@ -605,7 +605,7 @@ gtk_cell_renderer_text_finalize (GObject *object)
   if (priv->language)
     g_object_unref (priv->language);
 
-  (* G_OBJECT_CLASS (gtk_cell_renderer_text_parent_class)->finalize) (object);
+  G_OBJECT_CLASS (gtk_cell_renderer_text_parent_class)->finalize (object);
 }
 
 static PangoFontMask
@@ -1023,7 +1023,7 @@ gtk_cell_renderer_text_set_property (GObject      *object,
           priv->markup_set = FALSE;
         }
 
-      celltext->text = g_strdup (g_value_get_string (value));
+      celltext->text = g_value_dup_string (value);
       g_object_notify (object, "text");
       break;
 
@@ -1701,6 +1701,7 @@ gtk_cell_renderer_text_editing_done (GtkCellEditable *entry,
 {
   const gchar *path;
   const gchar *new_text;
+  gboolean canceled;
   GtkCellRendererTextPrivate *priv;
 
   priv = GTK_CELL_RENDERER_TEXT_GET_PRIVATE (data);
@@ -1725,9 +1726,12 @@ gtk_cell_renderer_text_editing_done (GtkCellEditable *entry,
       priv->entry_menu_popdown_timeout = 0;
     }
 
-  gtk_cell_renderer_stop_editing (GTK_CELL_RENDERER (data), 
-				  GTK_ENTRY (entry)->editing_canceled);
-  if (GTK_ENTRY (entry)->editing_canceled)
+  g_object_get (entry,
+                "editing-canceled", &canceled,
+                NULL);
+  gtk_cell_renderer_stop_editing (GTK_CELL_RENDERER (data), canceled);
+
+  if (canceled)
     return;
 
   path = g_object_get_data (G_OBJECT (entry), GTK_CELL_RENDERER_TEXT_PATH);
@@ -1801,7 +1805,9 @@ gtk_cell_renderer_text_focus_out_event (GtkWidget *entry,
   if (priv->in_entry_menu)
     return FALSE;
 
-  GTK_ENTRY (entry)->editing_canceled = TRUE;
+  g_object_set (entry,
+                "editing-canceled", TRUE,
+                NULL);
   gtk_cell_editable_editing_done (GTK_CELL_EDITABLE (entry));
   gtk_cell_editable_remove_widget (GTK_CELL_EDITABLE (entry));
 
@@ -1875,14 +1881,14 @@ gtk_cell_renderer_text_start_editing (GtkCellRenderer      *cell,
     }
 
   g_signal_connect (priv->entry,
-		    "editing_done",
+		    "editing-done",
 		    G_CALLBACK (gtk_cell_renderer_text_editing_done),
 		    celltext);
-  priv->focus_out_id = g_signal_connect_after (priv->entry, "focus_out_event",
+  priv->focus_out_id = g_signal_connect_after (priv->entry, "focus-out-event",
 					       G_CALLBACK (gtk_cell_renderer_text_focus_out_event),
 					       celltext);
   priv->populate_popup_id =
-    g_signal_connect (priv->entry, "populate_popup",
+    g_signal_connect (priv->entry, "populate-popup",
                       G_CALLBACK (gtk_cell_renderer_text_populate_popup),
                       celltext);
  
